//*****************************************************************************************
//                                                                            	          *
//		Test per EasyCAT PRO                                                   	          *  
//      VERSIONE 0.0    160608                                                	          *
//                                                                            	          *  
//*****************************************************************************************

#include <p18f26k22.h>
#include <delays.h> 
#include <stdio.h>

#include "main.h"
#include "easycat.h"



//--- cpu configuration bits -------------------------------------------------------------

#pragma	config	FOSC 	= INTIO67      		// Internal oscillator block, port function on RA6 and RA7
#pragma	config	PLLCFG  = ON                // Oscillator multiplied by 4 

#pragma	config  PRICLKEN = ON               // Primary clock enabled 
#pragma	config  FCMEN = OFF                 // Fail-Safe Clock Monitor disabled 
#pragma	config  IESO = OFF                  // Oscillator Switchover mode disabled 

#pragma	config  PWRTEN = ON                 // Power up timer enabled 
#pragma	config  BOREN = SBORDIS             // Brown-out Reset enabled in hardware only (SBOREN is disabled) 
#pragma	config  BORV = 285                  // BORV = 285 VBOR set to 2.85 V nominal 

#pragma	config  WDTEN = OFF                 // Watch dog timer is always disabled. SWDTEN has no effect. 

#pragma	config  MCLRE = EXTMCLR             // MCLR pin enabled, RE3 input pin disabled

#pragma	config  STVREN = ON                 // Stack full/underflow will cause Reset 
#pragma	config  LVP 	= OFF       		// Single-Supply ICSP disabled

#pragma	config  XINST 	= OFF       		// Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
#pragma	config  DEBUG   = OFF       		// Background debugger disabled, RB6 and RB7 configured as general purpose I/O pins



//--- variabili globali -----------------------------------------------------------------

BUFFOUT Out;
BUFFOUT* pBuffOut = &Out; 

BUFFIN In;
BUFFIN*  pBuffIn  = &In; 


//--- funzioni locali ------------------------------------------------------------------

void PicInit (void);
void InterruptHandlerHigh (void);



//-------------------------------------------------------------------------------------//
//																				       //
//		Main																      	   //
//																				       //			
//-------------------------------------------------------------------------------------//



//----- inizializzazione ---------------------------------------------------------------

void main (void)  

{
	PicInit();				                    // inizializza il PIC 

    if  (EasyCAT_Init() == false)               // se fallisce l'inizializzazione 
    {                                           // dell'EasyCAT resta in loop facendo
        while (1)                               // lampeggiare il led
        {                                       //
            Led = 1;                            //    
            Delay10KTCYx(200);                  // wait 125mS   
            Led = 0;                            //
            Delay10KTCYx(200);                  // wait 125mS
        } 
    }
            
    else                                        // tre lampi per segnalare EasyCAT inizializzata correttamente
    {                                           //    
        Led = 1;                                //    
        Delay10KTCYx(200);                      // wait 125mS   
        Led = 0;                                //
        Delay10KTCYx(200);                      // wait 125mS

        Led = 1;                                //    
        Delay10KTCYx(200);                      // wait 125mS   
        Led = 0;                                //
        Delay10KTCYx(200);                      // wait 125mS

        Led = 1;                                //    
        Delay10KTCYx(200);                      // wait 125mS   
        Led = 0;                                //
        Delay10KTCYx(200);                      // wait 125mS
    }


    T0CONbits.TMR0ON = 1;                       // fa partire timer 0 (watch dog)


//----- main loop ----------------------------------------------------------------------
   
    while(1)
    {
        TMR0H = 0x00;                                   // azzera timer 0 (watchdog)   
        TMR0L = 0x00;                                   // !!!!! rispettare la sequenza H L !!!!!! 

        Delay10KTCYx(20);                               // wait 12.5mS  -- tempo di ciclo 
       
        EasyCAT_MainTask();                             //--- scambio dati con l'interfaccia EtherCAT                                                   

        if (Out.Byte[0] & 0b00000001)                   // gestione byte di ouput
            Out_0 = 1;                                  //
        else                                            //
            Out_0 = 0;                                  //
        if (Out.Byte[0] & 0b00000010)                   //
            Out_1 = 1;                                  //
        else                                            //        
            Out_1 = 0;                                  //
        if (Out.Byte[0] & 0b00000100)                   //
            Out_2 = 1;                                  //
        else                                            //        
            Out_2 = 0;                                  //
        if (Out.Byte[0] & 0b00001000)                   //
            Out_3 = 1;                                  //
        else                                            //
            Out_3 = 0;                                  //
        if (Out.Byte[0] & 0b00010000)                   //
            Out_4 = 1;                                  //
        else                                            //
            Out_4 = 0;                                  //
        if (Out.Byte[0] & 0b00100000)                   //        
            Out_5 = 1;                                  //
        else                                            //    
            Out_5 = 0;                                  //
        if (Out.Byte[0] & 0b01000000)                   //
            Out_6 = 1;                                  //
        else                                            //
            Out_6 = 0;                                  //
        if (Out.Byte[0] & 0b10000000)                   //
            Out_7 = 1;                                  //
        else                                            //
            Out_7 = 0;                                  //

        In.Byte[6] = 0x00;                              // gestione byte di input    
        if (!In_0)                                      //
            In.Byte[6] |= 0b00000001;                   //
        if (!In_1)                                      //        
            In.Byte[6] |= 0b00000010;                   //    
        if (!In_2)                                      //
            In.Byte[6] |= 0b00000100;                   //
        if (!In_3)                                      //
            In.Byte[6] |= 0b00001000;                   //    
        if (!In_4)                                      //
            In.Byte[6] |= 0b00010000;                   //
        if (!In_5)                                      //
            In.Byte[6] |= 0b00100000;                   //
        if (!In_6)                                      //
            In.Byte[6] |= 0b01000000;                   //
        if (!In_7)                                      //
            In.Byte[6] |= 0b10000000;                   //

    
        ADCON0bits.CHS = 0b00000 ;                      // gestione ingresso analogico "0"
        ADCON0bits.DONE = 1;                            //
        while (ADCON0bits.DONE){};                      //        
        In.Byte[0] = ADRESL >> 2;                       //
        In.Byte[0] &= 0b00111111;                       //    
        In.Byte[0] |= 0b11000000 & (ADRESH << 6);       //


        ADCON0bits.CHS = 0b00001 ;                      // gestione ingresso analogico "1"
        ADCON0bits.DONE = 1;                            //
        while (ADCON0bits.DONE){};                      //
        In.Byte[1] = ADRESL >> 2;                       //
        In.Byte[1] &= 0b00111111;                       //
        In.Byte[1] |= 0b11000000 & (ADRESH << 6);       //     

        In.Byte[31]--;                                  // rampa di test
    }
}

//-------------------------------------------------------------------------------------//
//																				       //
//		Funzioni     														      	   //
//																				       //			
//-------------------------------------------------------------------------------------//


//------- Inizializzazione --------------------------------------------------------------
	
void PicInit (void)

{
											//---- configurazione CLOCK -----------------
											//


    OSCCON  = 0x00;                         //
    OSCCONbits.IRCF = 0b111;                // HFINTOSC Oscillator 16Mhz

    OSCTUNE = 0x00;
	OSCTUNEbits.PLLEN = 1;					// PLLx4 abilitato    fosc = 16MHz*4 = 64MHz 

    SLRCON = 0x00;                          // tutte le uscite con slew rate standard

    while (!OSCCONbits.HFIOFS){};           // aspetta che HFINTOSC sia stabile


											//---- configurazione porte di I/O ----------
                                            //
    ANSELA = 0b00000011;                    // ingressi analogici su RA0 e RA1               
    ANSELB = 0b00000000;                    // tutti gli altri pin sono digitali
    ANSELC = 0b00000000;                    //

    TRISA = 0b11111111;

    LATB  = 0b00000000;
    TRISB = 0b00000000;

    LATC  = 0b00000100;
    TRISC = 0b00010011; 

											//---- configurazione ADC -----------------------------
											//
    ADCON1 = 0x00;                          // VEF+ VDD   VREF- = VSS

    ADCON2 = 0x00;                          //
    ADCON2bits.ADFM = 1;                    // result is right justified
    ADCON2bits.ADCS = 0b101;                // AD clock fosc/16  = 16MHz/16 = 1MHz   
    ADCON2bits.ACQT = 0b111;                // acquisition time 20*Tad  = 20uS    

    ADCON0bits.ADON = 1;                    // ADC on


    										//---- configurazione SPI1 master-----------------------------
    										//
                                            // modalita' master usato per comunicazione con modulo EasyCAT PRO
                                            //                  
    SSP1STAT = 0x00;                        //
    SSP1STATbits.CKE = 1;                   //  transmit occurs on transition from active to idle clock state
                                            // (falling edge)        
    SSP1CON1 = 0x00;                        //    
    SSP1CON1bits.CKP = 0;                   // idle state for the clock is a low level 
                                            //
    SSP1CON1bits.SSPM = 0b0001;             // clock FOSC/16  = 64MHz/16 = 4MHz   
    SSP1CON1bits.SSPEN = 1;                 // enable SPI


    										//---- configurazione Timer 0 -----------------------------
    										//
                                            // usato per watch dog -- 260mS
                                            //    
    T0CON = 0x00;                           // internal instruction cycle clock
    T0CONbits.T0PS = 0b101;                 // prescaler /64
    T0CONbits.T08BIT = 0;                   // timer a 16 bit   
                                            //
    TMR0H = 0x00;                           // azzera timer 0    
    TMR0L = 0x00;                           // !!!!! rispettare la sequenza H L !!!!!! 

    INTCON2bits.TMR0IP = 1;                 // interrupt alta priorita'
	INTCONbits.TMR0IF = 0;				    // resetta il flag di interrupt 
    INTCONbits.TMR0IE = 1;                  // abilita interrupt


										    //---- configurazione INTERRUPT ---------------------
											//
	RCONbits.IPEN  = 1;						// abilita la gestione della priorita' interrupt
	INTCONbits.GIE = 1;						// abilita general interrupt

}



//--------------- High priority interrupt vector ------------------------------                                                
                                                                                 
#pragma code InterruptVectorHigh = 0x08                                          

void InterruptVectorHigh (void)                                                       
{                                                                                
  _asm                                                                           
    goto InterruptHandlerHigh 						//jump to interrupt routine                        
  _endasm                                                                        
}                                                                                


//----------------- High priority interrupt routine ---------------------------                                            
                                                                               
#pragma code                                                                   
#pragma interrupt InterruptHandlerHigh                                         
                                                                               
void InterruptHandlerHigh () 					    // timer 0 interrupt (watch dog 260mS) 
         
{                                             
    Led = 0;                                        // spegni il led di run    
                                                    // (e disabilita gli output)

	INTCONbits.TMR0IF = 0;							// resetta il flag di interrupt da Timer 0
}


